import java.util.*;
import java.io.*;

public class MakeSimpleDictionary { 
    static class Word { 
        String eng;
        String rus;
        int    count;
    };

    static final int MAX_FILE_SIZE = 32*1024;
    static final String OUTPUT = "..\\res\\";

    static byte[] toByteArray(String str) {
        int len = str.length();
        byte[] bytes = new byte[len];
        for (int i = 0; i < len; i++) { 
            bytes[i] = (byte)str.charAt(i);
        }
        return bytes;
    }

    static byte[] encode(String str) { 
        int len = str.length();
        byte[] buf = new byte[len*4+1];
        int charset = -1;
        int i, j, k;
        for (i = 0, j = 0, k = 0; i < len; i++) { 
            char ch = str.charAt(i);
            if ((ch >> 8) != charset || j - k == 0xFF + 2) { 
                buf[k] = (byte)(j-k-2);
                charset = (ch >> 8);
                k = j;
                buf[j+1] = (byte)charset;
                j += 2;
            }
            buf[j++] = (byte)ch;
        } 
        buf[k] = (byte)(j-k-2);
        buf[j++] = 0;
        byte[] result = new byte[j];
        System.arraycopy(buf, 0, result, 0, j);
        return result;
    }

    public static void main(String[] args) throws Exception {
        String dictionaryPath = args[0];
	BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(dictionaryPath), "utf-8"));
	String line;
	TreeMap hash = new TreeMap();
      nextLine:
	while ((line = in.readLine()) != null) { 
	    int i = line.indexOf(':');
	    if (i > 0) {
		Word word = new Word();                
		word.eng = line.substring(0, i).toLowerCase();
                word.rus = line.substring(i+1).trim();
                if (hash.get(word.eng) == null) { 
                    hash.put(word.eng, word);
                }
	    }
	}
	FileOutputStream index = new FileOutputStream(OUTPUT + "dictionary.idx");
	FileOutputStream out = new FileOutputStream(OUTPUT + "volume.1");
	Iterator iterator = hash.values().iterator();
	int n = 0;
	int size = 0;
	int totalSize = 0;
        int id = 1;
        String prevWord = null;
        StringBuffer ib = new StringBuffer();
	while (iterator.hasNext()) { 
	    Word word = (Word)iterator.next();
            StringBuffer sb = new StringBuffer();
            String thisWord = word.eng;
            if (prevWord == null) { 
                ib.append(thisWord);
                ib.append(':');
                sb.append(thisWord);
            } else { 
                int prefix;
                int len = thisWord.length() > prevWord.length() ? prevWord.length() : thisWord.length();
                if (len > 9) { 
                    len = 9;
                }
                for (prefix = 0; prefix < len && thisWord.charAt(prefix) == prevWord.charAt(prefix); prefix++);
                if (prefix > 0) { 
                    sb.append((char)(prefix + '0'));
                    sb.append(thisWord.substring(prefix));
                } else { 
                    sb.append(thisWord);
                }
            }
            sb.append(':');
            byte[] translation = encode(word.rus);
            //sb.append('\r');
            byte[] phrase = toByteArray(sb.toString());
            byte[] bytes = new byte[phrase.length + translation.length];            
            System.arraycopy(phrase, 0, bytes, 0, phrase.length);
            System.arraycopy(translation, 0, bytes, phrase.length, translation.length);
            if (size + bytes.length > MAX_FILE_SIZE) { 
                ib.append(prevWord);
                ib.append('\n');
                index.write(toByteArray(ib.toString()));
                ib = new StringBuffer();
                ib.append(thisWord);
                ib.append(':');
                out.close();
                out = new FileOutputStream(OUTPUT + "volume." + ++id);
                size = 0;
            }
            size += bytes.length;
            totalSize += bytes.length;
            n += 1;
            prevWord = thisWord;
            out.write(bytes);
	}
	out.close();
        ib.append(prevWord);
        ib.append('\n');
        index.write(toByteArray(ib.toString()));
        index.close();
	System.out.println("Words " + n + ", size " + totalSize);
    }
}